package mc600.boardgames;

import mc600.gamemanager.*;

/**
 * Insert the type's description here.
 * Creation date: (3/8/00 7:51:37 PM)
 * @author: 
 */
public class StickBoard {
	private int freelinescount;
	private int xdim;
	private int ydim;
	private int p1squares = 0;
	private int p2squares = 0;
	protected mc600.boardgames.StickSquare[][] quadrados;
/**
 * Insert the method's description here.
 * Creation date: (3/8/00 7:53:14 PM)
 * @param xdim int
 * @param ydim int
 */
public StickBoard(int xdim, int ydim) throws Exception{


	// seta dimensoes do tabuleiro 
	setXdim(xdim);
	setYdim(ydim);


	// determina numero de conexoes disponivel
	setFreelinescount( ( (getXdim()-1) * getYdim()) + ( (getYdim()-1) * getXdim() )   );


	// cria quadrados, que sao a composicao basica do tabuleiro
	quadrados = new StickSquare[(getXdim()-1)][(getYdim() -1)];
	
	for (int i=0; i < getXdim()-1; i++)
		for (int j=0; j < getYdim()-1; j++)
		quadrados[i][j] = new StickSquare();
			
	
	
	}
/**
 * Insert the method's description here.
 * Creation date: (3/24/00 7:48:37 PM)
 * @return int
 */
public int getFreelinescount() {
	return freelinescount;
}
/**
 * Insert the method's description here.
 * Creation date: (3/25/00 3:02:04 AM)
 * @return int
 */
public int getP1squares() {
	return p1squares;
}
/**
 * Insert the method's description here.
 * Creation date: (3/25/00 3:02:20 AM)
 * @return int
 */
public int getP2squares() {
	return p2squares;
}
/**
 * Insert the method's description here.
 * Creation date: (3/24/00 7:52:30 PM)
 * @return int
 */
public int getXdim() {
	return xdim;
}
/**
 * Insert the method's description here.
 * Creation date: (3/24/00 7:52:45 PM)
 * @return int
 */
public int getYdim() {
	return ydim;
}
/**
 * Retorna true caso o jogador tenha direito a mais um lance.
 *
 *
 * O tabuleiro eh formado por objetos do tipo quadrado, colocados lado a lado
 * Ao conectar dois pinos, sempre teremos que conectar uma aresta de dois quadrados.
 *    
 *     1    2    3    4  <-- X
 *   1 +----+----+----+
 *     | 1  | 2  | 3  |         O formato do lance deve ser  XxYxXyYy, dessa forma o lance "2232"
 *     |    |    |    |         ira provocar a conexao da aresta "c" do quadrado "2" e a aresta
 *   2 +----+----+----+         "a" do quadrado 5.
 *     | 4  | 5  | 6  |    
 *     |    |    |    |         Os objetos quadrado se encarregam de informar se foram fechados ou nao
 *   3 +----+----+----+         dando direito a um novo lance.
 *     | 7  | 8  | 9  |    
 *     |    |    |    |                   
 *   4 +----+----+----+
 *
 *   ^
 *   |
 *    Y
 *
 *  arestas do quadrado
 *
 *         a 
 *     o-------o
 *     |       |
 *   d |       | b
 *     |       |
 *     o-------o
 *         c
 *
 *
 * Creation date: (3/24/00 3:27:06 AM)
 * @return boolean
 * @param cast java.lang.String
 */
public boolean play(String cast, boolean isP1) throws InvalidCastException {

	int Xx,Yx,Xy,Yy; //coordenadas linha e coluna, x e y.

	//indice dos dois quadrados envolvidos em um lance
	int[] q1 = new int[2];
	int[] q2 = new int[2];

	// aresta do quadrado que sera fechada
	char a1,a2;

	boolean ganhoulanceA1=false;
	boolean ganhoulanceA2=false;
	

	if (cast.length() != Player.CASTSIZE) 

		throw new InvalidCastException("Tamanho de lance invalido");

	Xx = Integer.parseInt("0" + cast.charAt(0));
	Yx = Integer.parseInt("0" + cast.charAt(1));
	Xy = Integer.parseInt("0" + cast.charAt(2));
	Yy = Integer.parseInt("0" + cast.charAt(3));

	//verifica se cordenadas estao dentro do tabuleiro
	if ( (Xx > getXdim()) ||  (Xx < 1) || (Xy > getXdim()) ||  (Xy < 1) ||
		 (Yx > getYdim()) ||  (Yx < 1) || (Yy > getYdim()) ||  (Yy < 1))

		throw new InvalidCastException("Lance fora do tabuleiro <" 
					       + cast + "> ");

	//verifica se jogada e' valida
	if ( Math.abs(Xx-Xy) + Math.abs(Yx-Yy) != 1)

		throw new InvalidCastException("Lance invalido <" 
					       + cast + "> ");
	

	// determina quais quadrados estao envolvidos no lance em questao
	//
	// [nvaria][min(varia)]  e [nvaria-1][min(varia)]

	// determina qual nao varia
	if (Xx == Xy){
		//X nao varia
		q1[0] = Xx;
		q1[1] = java.lang.Math.min(Yy,Yx);

		q2[0] = Xx-1;
		q2[1] = java.lang.Math.min(Yy,Yx);

		// se x nao varia, aresta soh pode ser vertical (d ou b)
		// nesse caso, sera "b" para o quadrado de menor coordenada X (q2) e "d" para o de maior (q1).
		a2 = 'b';
		a1 = 'd';		
		
	}else{
		//Y nao varia
		q1[0] = java.lang.Math.min(Xy,Xx);
		q1[1] = Yy;

		q2[0] = java.lang.Math.min(Xy,Xx);
		q2[1] = Yy-1;

		// se y nao varia, aresta soh pode ser horizontal (a ou c)
		// nesse caso, sera "a" para o quadrado de menor coordenada Y (q2) e "c" para o de maior (q1).
		a2 = 'c';
		a1 = 'a';		
		
	}
	
	// conecta arestas, verificando se nao esta na borda do tabuleiro
	if ( (q1[0]>0) && (q1[1]>0) && (q1[0] < getXdim() ) && (q1[1] < getYdim() ))

		ganhoulanceA1 = quadrados[q1[0]-1][q1[1]-1].connect(isP1,a1);

	if ( (q2[0]>0) && (q2[1]>0) && (q2[0] < getXdim()) && (q2[1] < getYdim()) )

		ganhoulanceA2 = quadrados[q2[0]-1][q2[1]-1].connect(isP1,a2);
		
			
	// Atualiza numero de linhas diponiveis no tabuleiro
	setFreelinescount(getFreelinescount() - 1);


	// Atualiza numero de quadrados fechados por jogador
	if (ganhoulanceA1 || ganhoulanceA2)

		if (isP1)
			setP1squares(getP1squares() + (ganhoulanceA1?1:0) + (ganhoulanceA2?1:0) );
		else
			setP2squares(getP2squares() + (ganhoulanceA1?1:0) + (ganhoulanceA2?1:0) );

	
	return (ganhoulanceA1 || ganhoulanceA2);
}
/**
 * Insert the method's description here.
 * Creation date: (3/24/00 7:48:37 PM)
 * @param newFreelinecount int
 */
protected void setFreelinescount(int newFreelinescount) {

	if (newFreelinescount >= 0)
		freelinescount = newFreelinescount;
}
/**
 * Insert the method's description here.
 * Creation date: (3/25/00 3:02:04 AM)
 * @param newP1squares int
 */
protected void setP1squares(int newP1squares) {
	p1squares = newP1squares;
}
/**
 * Insert the method's description here.
 * Creation date: (3/25/00 3:02:20 AM)
 * @param newP2squares int
 */
protected void setP2squares(int newP2squares) {
	p2squares = newP2squares;
}
/**
 * Insert the method's description here.
 * Creation date: (3/24/00 7:52:30 PM)
 * @param newXdim int
 */
private void setXdim(int newXdim) throws Exception{

	if (newXdim < 2)
		throw new Exception("Dimensao do tabuleiro invalida.\n");
		
	xdim = newXdim;
}
/**
 * Insert the method's description here.
 * Creation date: (3/24/00 7:52:45 PM)
 * @param newYdim int
 */
private void setYdim(int newYdim) throws Exception{

	if (newYdim < 2)
		throw new Exception("Dimensao do tabuleiro invalida.\n");
	
	ydim = newYdim;
}
}
