package mc600.gamemanager;

import mc600.boardgames.*;
import java.io.*;
/**
 * Insert the type's description here.
 * Creation date: (3/8/00 9:24:49 PM)
 * @author: 
 */
public class Judge {
	private PlayerClient p1,p2;
	private mc600.boardgames.StickBoard board;
/**
 * Construtor do juiz.
 *
 * Este objeto repassa os comandos dos jogadores para o tabuleiro,
 * controlando o timeout para cada lance, apos o termino do jogo,
 * incrementa a contagem de vitorias do jogador vencedor.
 *
 * Creation date: (3/8/00 9:26:09 PM)
 * @param p1 mc600.gamemanager.PlayerClient
 * @param p2 mc600.gamemanager.PlayerClient
 */
public Judge(PlayerClient p1, PlayerClient p2) {

	this.p1 = p1;
	this.p2 = p2;

	// verifica se jogadores sao validos (um deles pode ser jogador de paridade)
	if ( (p1!=null) && (p2!= null) ){

	
		// "restarta" jogadores para nova partida
		// se chegou aqui eh porque os jogadores foram criados,
		// logo, se houver excessao significa que jogador caiu por algum motivo.
		// Perde por WO.
		try{
			if (!p1.restart())
				p2.setVictorycount(p2.getVictorycount() + 1);
				
			if (!p2.restart())
				p1.setVictorycount(p1.getVictorycount() + 1);


			// inicializa tabuleiro
			board = new StickBoard(Player.XDIM,Player.YDIM);

			// faz jogo
			this.run();
					
		}catch (Exception e){

			System.out.println("Erro: " + e.getMessage());
		}

	}
		
		
	
	}
/**
 *    Grava arquivo "<nomejogador>.log" com mensagem "msg"
 *    Se arquivo ja existe, apenda.
 *
 */
private void log(PlayerClient p, String msg){

	try{
	// Abre o arquivo 
	FileWriter out = new FileWriter (p.getPlayername()+".log", false);

	out.write(msg + "\n");
	out.flush();

	out.close();
	}catch (Exception e){

		System.out.println("Juiz: Erro ao gravar log.");

	}

}
/**
 * Funcao principal do Juiz
 *
 *
 * O objeto juiz eh encarregado de solicitar os lances aos jogadores, e repassa-los para	
 * o tabuleiro, que decide sobre a validade dos mesmos.
 *
 * Existem 3 maneiras de o jogo ser encerrado:
 * 
 *  1- O jogador efetua um lance invalido
 *  2- O jogador demora mais que "Player.MAXRESPONSETIME" para jogar
 *  3- O jogador provoca um erro de IO ao enviar o lance.
 *
 * Nos tres casos o infrator perde a partida. Sera gravado um log do juiz informando-lhe do
 * ocorrido. :)
 *
 * Se o jogo terminar normalmente, o vencedor eh determinado pelo numero de quadrados que fechou,
 * o que tiver o maior vence.
 *
 */
public void run() {

	PlayerClient pdavez=null;   // define jogador que possue o lance atual
	boolean ganhoulance=false;  // flag indicando se jogador ganhou outro lance
	String  cast=null;			// variavel para armazenar o lance retornado pelo jogador

		
	// verifica se os dois jogadores estao alocados
	if ( (p1 != null) && (p2 != null) ){

			
		System.out.println("Juiz: Iniciando partida <" + p1.getPlayername() + "> X <" + p2.getPlayername() + "> ");

		try{

			// pdavez define o jogador atualmente jogando
			pdavez = p1;
			// Primeiro lance deve ser comando para jogador iniciar partida
			cast = Player.STARTCMD;
					
			// continua enquanto houver linhas livres
			while (board.getFreelinescount() != 0){
				
				// manda lance para jogador "da vez", recebendo replica
				cast = pdavez.play(cast);
				// passa replica para o tabuleiro
				ganhoulance = board.play(cast,pdavez.equals(p1)?true:false);
				
				// se jogador fechou quadrado, ganhou mais um lance, portando
				// o jogador "da vez" parmanece o mesmo.
				if (!ganhoulance)
					pdavez = pdavez.equals(p1)?p2:p1;

				else{
					// de qualquer forma, o oponente deve saber qual foi o lance.
					
					if (pdavez.equals(p1))
						p2.play(cast);
					else
						p1.play(cast);
				}
				
			} // end while getfreelinescount
				
  
		}catch (InvalidCastException e){


	  
			// grava log...
			log(pdavez,"Juiz: Partida <" + p1.getPlayername() + "> X <" + p2.getPlayername() + "> ");
			log(pdavez,"Juiz: Jogador <" + pdavez.getPlayername() + "> submeteu lance invalido: <"+cast + ">!!");
		}catch (IOException e){

			// grava log
			log(pdavez,"Juiz: Partida <" + p1.getPlayername() + "> X <" + p2.getPlayername() + "> ");
			log(pdavez,"Juiz: Erro de io: " + e.getMessage() + " provocado por: <" + pdavez.getPlayername() + ">!");

		}catch (Exception e){

			// grava log
			log(pdavez,"Juiz: Partida <" + p1.getPlayername() + "> X <" + p2.getPlayername() + "> ");
			log(pdavez,"Juiz: Erro geral: " + e.getMessage() + " provocado por: <" + pdavez.getPlayername() + ">!");

		}


		////// Fim do jogo, determina quem foi o vencedor ///////////////////

		if (board.getFreelinescount() == 0){
			// jogo acabou normalmente, tabuleiro decide quem venceu
			if (board.getP1squares() > board.getP2squares())
				// p1 venceu
				p1.setVictorycount(p1.getVictorycount() + 1);
			else if (board.getP1squares() < board.getP2squares())
				// p2 venceu
				p2.setVictorycount(p2.getVictorycount() + 1);
			else{
				// empate (pode ocorrer jah que as dimensoes do tabuleiro sao variaveis...
				p1.setVictorycount(p1.getVictorycount() + 1);			
				p2.setVictorycount(p2.getVictorycount() + 1);
			}

			
		}
		else{
			// jogo terminou devido a excessao, quem gerou perde....

			if (pdavez.equals(p1))
				p2.setVictorycount(p2.getVictorycount() + 1);
			else
				p1.setVictorycount(p1.getVictorycount() + 1);

		}
	///////////////////////////////////////////////////////////
 	 try{			
		p1.shutdown();
		p2.shutdown();
	 }catch (IOException e){
			System.out.println("Erro ao finalizar jogadores: " + e.getMessage());
	 }
  	

	System.out.println("Juiz: Partida <" + p1.getPlayername() + "> X <" + p2.getPlayername() + "> encerrada.");

	
} // end if p != null

	
}
}
