package mc600.gamemanager;

import java.io.*;
import java.net.*;
/**
 * Insert the type's description here.
 * Creation date: (3/8/00 11:04:48 PM)
 * @author: 
 */
public class PlayerClient extends Player{
	private Socket skt;
		private int victorycount = 0;
	private long timeleft = MAXGAMETIME;
	private InetAddress addr = null;
	private int port = 0;
/**
 * Tenta criar cliente do jogador no addr, porta port...
 *
 * @param port int
 * @param email java.lang.String
 * @exception java.lang.Exception The exception description.
 */
public PlayerClient(InetAddress addr, int port) throws Exception {
	super();


	this.port = port;
	this.addr = addr;


	// Tenta conectar, se a porta nao esta alocada, levanta excesao
	skt = new Socket (addr, port);

//	skt.setTcpNoDelay(true);	


	// Valida jogador, se tudo correr bem, nome do jogador sera registrado e o objeto criado
	setPlayername(validatePlayer(skt));
	

}
/**
 * Insert the method's description here.
 * Creation date: (3/24/00 12:12:44 AM)
 * @return int
 */
public int getVictorycount() {
	return victorycount;
}
private boolean hasTimetoPlay(){

	if (timeleft <= 0)
		return false;

	return true;
}
/**
 * Funcao recebe lance "cast", passando para o server, e aguardando resposta.Bloqueante.
 *
 * Creation date: (3/22/00 11:55:51 PM)
 * @return java.lang.String
 * @param cast java.lang.String
 * @exception java.lang.Exception The exception description.
 */
public String play(String cast) throws IOException {
   
		byte[] blance = new byte[4];
		long elapsedtime=0;


	if (!hasTimetoPlay())

	   throw new IOException("Excedido tempo maximo de jogo");

	// captura hora atual
	elapsedtime =  System.currentTimeMillis();

	// ajusta o timeout do socket para derrubar jogador caso demore muito...
	skt.setSoTimeout(Player.MAXRESPONSETIME);
	   	
	DataOutputStream dataOut = new DataOutputStream (skt.getOutputStream ());
	DataInputStream dataIn = new DataInputStream (skt.getInputStream ());

	// Envia ao servidor o lance
	dataOut.writeBytes(cast);
	dataOut.flush ();
		
	// Recebe lance do servidor
	dataIn.readFully(blance);


	// desliga o timeout
	skt.setSoTimeout(0);

	// calcula tempo decorrido
	elapsedtime = System.currentTimeMillis() - elapsedtime;
	timeleft -= elapsedtime;
		

	// Retorna resposta do servidor
	return (new String(blance));
}
/**
 * Reinicia socket, verificando se o mesmo jogador continua na porta
 * inicialmente alocada.
 * Caso positivo, retorna true. caso contrario, false.
 *
 * Creation date: (3/25/00 2:07:20 AM)
 * @exception java.lang.Exception The exception description.
 */
public boolean restart() throws Exception {

	String novonome;

	timeleft = Player.MAXGAMETIME;
	
	// Tenta conectar, se a porta nao esta alocada, levanta excesao
	skt = new Socket (addr, port);

	skt.setTcpNoDelay(true);
	
	novonome = validatePlayer(skt);

	// Valida jogador, e verifica se nome nao mudou.
	if (novonome.compareTo(getPlayername()) != 0)
		// nome do cliente mudou.... 
		return false;


	return true;
	}
/**
 * Insert the method's description here.
 * Creation date: (3/24/00 12:12:44 AM)
 * @param newVictorycount int
 */
protected void setVictorycount(int newVictorycount) {
	victorycount = newVictorycount;
}
/**
 * Insert the method's description here.
 * Creation date: (3/23/00 12:07:50 AM)
 * @exception java.io.IOException The exception description.
 */
public void shutdown() throws IOException {
	
	
	
	skt.close();

	
	}
/**
 * Verifica se o servidor alocado em "skt" eh valido.
 * Se for, retorna nome, senao, levanta excessao.
 *
 * Creation date: (3/25/00 2:09:43 AM)
 * @return java.lang.String
 */
private String validatePlayer(Socket skt) throws Exception{

	int i;
	byte[] buffer = new byte[MAGIC.length()];
	String magic;
	
	// ajusta o timeout do socket para derrubar jogador caso demore muito...
	skt.setSoTimeout(Player.MAXRESPONSETIME);
	
	DataInputStream dataIn = new DataInputStream (skt.getInputStream ());

	// Verifica se servidor eh valido, lendo MAGIC
	dataIn.readFully(buffer,0,MAGIC.length());
	magic = new String(buffer);
	
	if (magic.compareTo(MAGIC) != 0)

		throw new Exception("Servidor invalido.");

	
	// le o tamanho do nome do jogador
	i = dataIn.readInt();
	if ( (i <= 0) || (i>MAXPLAYERNAME) )

		throw new Exception("Tamanho do nome do jogador invalido.");
	
	// le o nome do jogador
	buffer = new byte[i];
	dataIn.readFully(buffer);
	
	// desliga o timeout
	skt.setSoTimeout(0);
	
	return (new String(buffer));
}
}
