package mc600.gamemanager;

import java.io.*;
import java.net.*;
/**
 * Este objeto representa o jogador. Os lances devem ser passados a ele.
 *
 * O procedimento eh simples, para iniciar o seu servidor voce deve seguir os seguintes passos:
 *
 *
 * 1 - Deve criar uma instancia do objeto PlayerServer, passando como parametro o seu e-mail
 * 
 * 2 - No corpo do seu programa voce deve incluir uma chamada para "waitClient()", que colocara
 *     o seu programa numa porta IP, escolhida automaticamente, esperando pela chamada do Gerenciador.
 *     apareca.
 * 
 * 3 - Ao chegar o Gerenciador, a funcao waitClient recebera o primeiro lance da partida, ou o comando de
 *     "STRT" (Player.STARTCMD), significando que voce eh o primeiro a jogar.
 *
 * 4 - Voce tem um tempo determinado por "Player.MAXRESPONSETIME" para decidir qual o seu lance, e entao
 *     repassa-lo para o servidor, para isto voce deve utilizar o comando "play(lance)", passando o seu
 *     lance como argumento. Esta funcao ira enviar o seu lance para o oponente, retornando a sua resposta.
 * 
 *
 * Caso voce submeta um lance invalido, ou demore para responder, a conexao sera derrubada, e voce perde
 * automaticamente a partida (nao o campeonato).
 *
 * Nos dois casos voce vai receber um e-mail do gerenciador, informando qual o motivo exato.
 *
 * Ao final do jogo a conexao sera derrubada tambem, para o proximo jogo sera iniciada outra.
 *
 * ATENCAO: Voce deve fazer um "loop" principal, que sempre chame a funcao "waitClient" apos o fechamento
 *          da conexao atual, caso contrario o seu jogador soh ira jogar a primeira partida!!
 *
 *
 * EXEMPLO: O Objeto "ServerTest" ilustra o corpo de um programa que faz essas coisas...
 *
 *
 *
 *
 * Creation date: (3/8/00 11:09:14 PM)
 * @author: Celso Cardoso
 */
public class PlayerServer extends Player {
	private java.net.ServerSocket sktServer;
	private Socket skt;
/**
 * PlayerServer constructor comment.
 * @param email java.lang.String
 * @exception java.lang.Exception The exception description.
 */
public PlayerServer(int port, String email) throws Exception {
	super(email);


	if ( (port > Player.MAXIP) || (port < Player.MINIP) )

		throw new Exception("PlayerClient: Porta Invalida");

	// Cria o socket servidor na porta especificada
	sktServer = new ServerSocket (port);

	
}
/**
 * Insert the method's description here.
 * Creation date: (3/25/00 3:49:58 AM)
 * @param email java.lang.String
 */
public PlayerServer(String email) throws Exception{
	
	super(email);

	int port;

	// Encontra primeira porta livre... Q&D

	for (port=MINIP; port<=MAXIP; port++){

		try{

			// Cria o socket servidor na porta especificada
			sktServer = new ServerSocket (port);
			break;

		}catch (Exception e){} // ignora excessoes... esta apenas procurando porta livre..

	}
		

	if (port > MAXIP)
		// nao encontrou porta livre!!
		throw new Exception("Nenhuma porta livre encontrada!! Chame o Bene!");

	
	}
/**
 * Funcao utilizada para enviar e receber os lances do servidor
 * Para o jogo dos risquinhos, o tabuleiro sera da seguinte forma:
 *
 *
 *
 * O tabuleiro eh formado por objetos do tipo quadrado, colocados lado a lado
 * Ao conectar dois pinos, sempre teremos que conectar uma aresta de dois quadrados.
 *    
 *     1    2    3    4  <-- X
 *   1 +----+----+----+
 *     | 1  | 2  | 3  |         O formato do lance deve ser  XxYxXyYy, dessa forma o lance "2232"
 *     |    |    |    |         ira provocar a conexao da aresta "c" do quadrado "2" e a aresta
 *   2 +----+----+----+         "a" do quadrado 5.
 *     | 4  | 5  | 6  |    
 *     |    |    |    |         Os objetos quadrado se encarregam de informar se foram fechados ou nao
 *   3 +----+----+----+         dando direito a um novo lance.
 *     | 7  | 8  | 9  |    
 *     |    |    |    |                   
 *   4 +----+----+----+
 *
 *   ^
 *   |
 *    Y
 *
 *  arestas do quadrado
 *
 *         a 
 *     o-------o
 *     |       |
 *   d |       | b
 *     |       |
 *     o-------o
 *         c
 *
 * Se o jogador tiver direito a um outro lance adicional, a funcao retornara o seu lance anterior,
 * aguardando o proximo.
 *
 * Creation date: (3/23/00 1:03:54 AM)
 * @return java.lang.String
 * @param cast java.lang.String
 * @exception java.lang.Exception The exception description.
 */
public String play() throws Exception {

	return play("0000");
}
/**
 * Funcao utilizada para enviar e receber os lances do servidor
 * Para o jogo dos risquinhos, o tabuleiro sera da seguinte forma:
 *
 *
 *
 * O tabuleiro eh formado por objetos do tipo quadrado, colocados lado a lado
 * Ao conectar dois pinos, sempre teremos que conectar uma aresta de dois quadrados.
 *    
 *     1    2    3    4  <-- X
 *   1 +----+----+----+
 *     | 1  | 2  | 3  |         O formato do lance deve ser  XxYxXyYy, dessa forma o lance "2232"
 *     |    |    |    |         ira provocar a conexao da aresta "c" do quadrado "2" e a aresta
 *   2 +----+----+----+         "a" do quadrado 5.
 *     | 4  | 5  | 6  |    
 *     |    |    |    |         Os objetos quadrado se encarregam de informar se foram fechados ou nao
 *   3 +----+----+----+         dando direito a um novo lance.
 *     | 7  | 8  | 9  |    
 *     |    |    |    |                   
 *   4 +----+----+----+
 *
 *   ^
 *   |
 *    Y
 *
 *  arestas do quadrado
 *
 *         a 
 *     o-------o
 *     |       |
 *   d |       | b
 *     |       |
 *     o-------o
 *         c
 *
 * Se o jogador tiver direito a um outro lance adicional, a funcao retornara o seu lance anterior,
 * aguardando o proximo.
 *
 * Creation date: (3/23/00 1:03:54 AM)
 * @return java.lang.String
 * @param cast java.lang.String
 * @exception java.lang.Exception The exception description.
 */
public String play(String cast) throws Exception {

	byte[] bcast = new byte[CASTSIZE];

	
	if (cast.length() != CASTSIZE)

		throw new Exception("Tamanho de lance invalido.");


	if (skt == null)
		
		throw new Exception("Socket nao inicializado.");


	// Cria os buffers de saida e entrada
	DataInputStream dataIn = new DataInputStream (skt.getInputStream ());
   	DataOutputStream dataOut = new DataOutputStream (skt.getOutputStream ());

   	// Envia o lance
   	dataOut.writeBytes(cast);
   	dataOut.flush();
	   	   
 		  	   	
	 // Le o proximo comando
	dataIn.readFully(bcast);
	
		
	return (new String(bcast));			

}
	public void shutdown() throws IOException {


		skt.close();


	}
/**
 * Esta funcao aguarda cliente, retornando string contendo:
 *
 *  "STRT" : caso o primeiro lance seja deste jogador
 *  "XYXY" : coordenada do primeiro lance. caso este jogador seja o segundo.
 *
 *
 * Creation date: (3/23/00 1:05:20 AM)
 * @return java.lang.String
 * @exception java.lang.Exception The exception description.
 */
public String waitClient() throws Exception {

	   byte[] cast = new byte[CASTSIZE];

	   
	   // Cria socket para intercambio dos dados
	   skt = sktServer.accept (); 

	   skt.setTcpNoDelay(true);
		
	   // Cria os buffers de saida e entrada
	   DataInputStream dataIn = new DataInputStream (skt.getInputStream ());
   	   DataOutputStream dataOut = new DataOutputStream (skt.getOutputStream ());

   	   
   	   // Envia o Magic..
   	   dataOut.writeBytes(MAGIC);

   	   // Envia o tamanho do playername
   	   dataOut.writeInt(getPlayername().length());

   	   // Envia o nome do jogador
   	   dataOut.writeBytes(getPlayername());
   	   dataOut.flush();
	   	   
	   // Le o proximo comando
	   dataIn.readFully(cast,0,CASTSIZE);
	
		
	return (new String(cast));
}
}
